#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <GL/glut.h>
#include <math.h>
#include "mcTexGen.hpp"

class SkyTexture : public mcTexGen
{
  private:
	int r1, g1, b1, r2, g2, b2;
	int r3, g3, b3, r4, g4, b4;
  public:
	SkyTexture();
	virtual int getPixel(double x, double y);
	virtual void getPixels(GLubyte *image, double nwx, double nwy, double sex, double sey, int width, int height);
};

SkyTexture::SkyTexture()
{
	r1 = 45; g1 = 40; b1 = 178;
	r2 = 30; g2 = 50; b2 = 198;

	r3 = 245; g3 = 245; b3 = 255;
	r4 = 230; g4 = 230; b4 = 240;

}

int SkyTexture::getPixel(double x, double y)
{
	double val;
	
	selectPerlinOctave(0);
	val = hperlin(x / 16.4, y / 16.4);
	val += hperlin(x / 8.2, y / 8.2) / 2.0 ;
	val += hperlin(x / 4.1, y / 4.1) / 4.0;
	val += hperlin(x / 2.05, y / 2.05) / 8.0;
	val += hperlin(x, y) / 16.0;
	val /= 1.8;

	int ra = (r1 * val + r2 * (1.0 - val));
	int ga = (g1 * val + g2 * (1.0 - val));
	int ba = (b1 * val + b2 * (1.0 - val));
	int rb = (r3 * val + r4 * (1.0 - val));
	int gb = (g3 * val + g4 * (1.0 - val));
	int bb = (b3 * val + b4 * (1.0 - val));

	selectPerlinOctave(1);
	val = hperlin(x / 64.4, y / 64.4);
	val += hperlin(x / 32.1, y / 32.1) / 2.0;
	val += hperlin(x / 16.2, y / 16.2) / 3.5;
	val += hperlin(x / 8.1, y / 8.1) / 4.0;
	val += hperlin(x / 4.1, y / 4.1) / 8.0;
	val += hperlin(x / 2.1, y / 2.1) / 16.0;
	val /= 1.9;
	if (val > 1.0)
		val = 1.0;
	
	val = gain(val, 0.9);

	int r = (rb * val + ra * (1.0 - val));
	int g = (gb * val + ga * (1.0 - val));
	int b = (bb * val + ba * (1.0 - val));

	return (int)((r << 16) + (g << 8) + b);
}

void SkyTexture::getPixels(GLubyte *image, double nwx, double nwy, double sex, double sey, int width, int height)
{
	double dx = (sex - nwx) / (double)width, 
		   dy = (sey - nwy) / (double)height;
	double xc = nwx, 
		   yc = nwy;
	for (int y = 0; y < height; y++)
	{
		xc = nwx;
		for (int x = 0; x < width; x++)
		{
			/* Haetaan pikselin vri, voitaisiin optimoida 2d:t varten oma koodi */
			int px = getPixel(xc, yc);

			*image++ = (px & 0xFF0000) >> 16;
			*image++ = (px & 0xFF00) >> 8;
			*image++ = px & 0xFF;

			xc += dx;
		}
		yc += dy;
	}
}


class HeightTexture : public mcTexGen
{
  public:
	virtual int getPixel(double x, double y);
	virtual void getPixels(GLubyte *image, double nwx, double nwy, double sex, double sey, int width, int height);
};

int HeightTexture::getPixel(double x, double y)
{
	double val = hperlin(x / 16.4, y / 16.4);
	val += hperlin(x / 8.2, y / 8.2) / 2.0 ;
	val	+= hperlin(x / 4.1, y / 4.1) / 4.0;
	val += hperlin(x / 2.05, y / 2.05) / 8.0;
	val += hperlin(x, y) / 16.0;
	val /= 1.8;

	return (int)(val * 255);
}

void HeightTexture::getPixels(GLubyte *image, double nwx, double nwy, double sex, double sey, int width, int height)
{
	double dx = (sex - nwx) / (double)width, 
		   dy = (sey - nwy) / (double)height;
	double xc = nwx, 
		   yc = nwy;
	selectPerlinOctave(0);
	for (int y = 0; y < height; y++)
	{
		xc = nwx;
		for (int x = 0; x < width; x++)
		{
			/* Haetaan pikselin vri, voitaisiin optimoida 2d:t varten oma koodi */
			double val = hperlin(x / 16.4, y / 16.4);
			val += hperlin(x / 8.2, y / 8.2) / 2.0 ;
			val	+= hperlin(x / 4.1, y / 4.1) / 4.0;
			val += hperlin(x / 2.05, y / 2.05) / 8.0;
			val += hperlin(x, y) / 16.0;
			val /= 1.8;
			int height = val * 255;

			*image++ = height;
			*image++ = height;
			*image++ = height;

			xc += dx;
		}
		yc += dy;
	}

}
/*
*	void Render()
*
*	Called by GLUT whenever the screen needs to be drawn.	
*/

void Render()
{
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	/* Display the result */

    glBegin(GL_QUADS);
    glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0, 0.0);
    glTexCoord2f(0.0, 1.0); glVertex3f(-1.0, 1.0, 0.0);
    glTexCoord2f(1.0, 1.0); glVertex3f(1.0, 1.0, 0.0);
    glTexCoord2f(1.0, 0.0); glVertex3f(1.0, -1.0, 0.0);

    glEnd();
    glFlush();

	glutSwapBuffers();
}


/*
*	void Reshape(int w, int h)
*
*	Called at startup and whenever our window has been resized. 
*	Initializes the projection matrix so that the view does not get
* 	distorted when the screen size is changed. 
*
*	Naturally called only once when in fullscreen mode.
*/

void Reshape(int w, int h)
{
    glViewport(0, 0, w, h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60.0, 1.0*(GLfloat)w/(GLfloat)h, 1.0, 30.0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslatef(0.0, 0.0, -2.0);
}

int main(int argc, char *argv[])
{
	GLubyte *image = new GLubyte[256 * 256 * 3];

	/* Generoidaan texture */
	mcTexGen *gen = new HeightTexture();
	gen->getPixels(image, 0, 0, 255, 255, 256, 256);

	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutCreateWindow("TexGen");
	glutReshapeWindow(800, 600);
	glutDisplayFunc(Render);
	glutReshapeFunc(Reshape);
	glutIdleFunc(Render);

    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glTexImage2D(GL_TEXTURE_2D, 0, 3, 256, 256, 0, GL_RGB, GL_UNSIGNED_BYTE, image);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

    glEnable(GL_TEXTURE_2D);
   	glEnable(GL_DEPTH_TEST);
	glShadeModel(GL_FLAT);
	
	glutMainLoop();
}
